package build

import (
	"git.akyoto.dev/cli/q/src/build/cpu"
	"git.akyoto.dev/cli/q/src/build/expression"
)

// CompileFunctionCall executes a function call.
func (f *Function) CompileFunctionCall(expr *expression.Expression) error {
	var (
		funcName   = expr.Children[0].Token.Text()
		parameters = expr.Children[1:]
		isSyscall  = funcName == "syscall"
		registers  []cpu.Register
	)

	if isSyscall {
		registers = f.cpu.Syscall
	} else {
		registers = f.cpu.Call
	}

	registers = registers[:len(parameters)]

	for _, register := range registers {
		f.SaveRegister(register)
	}

	err := f.ExpressionsToRegisters(parameters, registers)

	if isSyscall {
		f.assembler.Syscall()
	} else {
		f.assembler.Call(funcName)
	}

	return err
}
